///§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§//*
//             LOW ENERGY MUON SPIN RELAXATION, ROTATION, RADIATION 
//
//  ID    : LEMuSRPgaMessenger.cc , v 1.2
//  AUTHOR: Taofiq PARAISO
//  DATE  : 2004-09-16 09:12
//§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§//
//
//  &           &&&&&&&&&&                            &&&&&&&         &&&&&&&&
//  &           &                                   &&      &&       &       &&
//  &           &                   &      &       &                &       &&
//  &           &&&&&&&            &      &         &&&&&&         &&&&&&&&  
//  &           &                 &      &&                &      &      &&
//  &           &                &&     &  &     &&      &&      &        &
//  &&&&&&&&&&  &&&&&&&&&&      &  &&&&&    &&    &&&&&&&       &        &&   
//                             &
//                            &
//                           &
//                         &  
//                          PRIMARY GENERATOR ACTION MESSENGER
//§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§§//


#ifndef LEMUSRPGAMESSENGER_H
#define LEMUSRPGAMESSENGER_H 1

class LEMuSRPrimaryGeneratorAction;
class G4UIdirectory;
class G4UIcmdWithoutParameter;
class G4UIcmdWithAString;
class G4UIcmdWithADoubleAndUnit;
class G4UIcmdWithADouble;
class G4UIcmdWith3Vector;
class G4UIcmdWith3VectorAndUnit;
class G4UIcmdWithAnInteger;

#include "G4UImessenger.hh"
#include "globals.hh"

class LEMuSRPgaMessenger: public G4UImessenger
{
  public:
    LEMuSRPgaMessenger(LEMuSRPrimaryGeneratorAction* thPGA);
    ~LEMuSRPgaMessenger();
    
  public:
    void SetNewValue(G4UIcommand * command,G4String newValues);

  private:
    LEMuSRPrimaryGeneratorAction * lemuPGA;
    
  private: //commands
  G4UIdirectory *             pgaDirectory;
  G4UIcmdWithAString          *setRndEnergy;
  G4UIcmdWithAString          *setMuonium;
  G4UIcmdWithoutParameter *reset;
  G4UIcmdWith3Vector *posCmd, *momCmd, *scaCmd, *scbCmd, *scgCmd, *setEGauss;    
  G4UIcmdWithADouble *setEnergy;
  G4UIcmdWithADouble *setKEoffset;
  G4UIcmdWithADouble *setCharge;




};

#endif
